//
//      coded by un
//            --------------
//                     mindshifter.com
//

using System;
using System.Collections.Generic;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;

namespace Mindshifter
{
    public class Sandbox : Game
    {
        private class KeyInputEvent
        {
            public Keys                         Key;
            public DiscreteInputEventType       Type;

            public KeyInputEvent(Keys key, 
                DiscreteInputEventType type)
            {
                Key = key;
                Type = type;
            }
        }

        private class MouseInputEvent
        {
            public MouseButtons                 Button;
            public DiscreteInputEventType       Type;

            public MouseInputEvent(MouseButtons button,
                DiscreteInputEventType type)
            {
                Button = button;
                Type = type;
            }
        }

        private GraphicsDeviceManager   mGraphics;
        private SpriteBatch             mSpriteBatch;
        private SpriteFont              mFont;

        private int                     mMouseX;
        private int                     mMouseY;
        private int                     mMouseZ;
        private int                     mMouseDX;
        private int                     mMouseDY;
        private int                     mMouseDZ;

        private List<KeyInputEvent>     mKeyEvents;
        private List<MouseInputEvent>   mMouseEvents;

        public Sandbox()
        {
            mGraphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            mKeyEvents = new List<KeyInputEvent>();
            mMouseEvents = new List<MouseInputEvent>();
        }

        protected override void Initialize()
        {
            InputManager.Initialize(this);

            IInputManager input = (IInputManager)Services.GetService(typeof(IInputManager));

            input.ActionMap.Bind(Keys.Escape, DiscreteInputEventType.Down,
                delegate(GameTime time)
                {
                    Exit();
                });

            input.ActionMap.MouseMoved += delegate(int dx, int dy, GameTime time)
            {
                mMouseDX = dx;
                mMouseDY = dy;
            };

            input.ActionMap.MouseScrolled += delegate(int dw, GameTime time)
            {
                mMouseDZ = dw;
            };

            for (int i = 0; i < 256; ++i)
            {
                Keys? key = (Keys)i;
                if (!key.HasValue)
                    continue;

                Keys val = key.Value;

                input.ActionMap.Bind(val, DiscreteInputEventType.Down,
                    delegate(GameTime time)
                    {
                        mKeyEvents.Add(new KeyInputEvent(val, DiscreteInputEventType.Down));
                    });

                input.ActionMap.Bind(val, DiscreteInputEventType.Up,
                    delegate(GameTime time)
                    {
                        mKeyEvents.Add(new KeyInputEvent(val, DiscreteInputEventType.Up));
                    });
            }

            for (int i = 0; i < 3; ++i)
            {
                MouseButtons button = (MouseButtons)i;

                input.ActionMap.Bind(button, DiscreteInputEventType.Down,
                    delegate(GameTime time)
                    {
                        mMouseEvents.Add(new MouseInputEvent(button, DiscreteInputEventType.Down));
                    });

                input.ActionMap.Bind(button, DiscreteInputEventType.Up,
                    delegate(GameTime time)
                    {
                        mMouseEvents.Add(new MouseInputEvent(button, DiscreteInputEventType.Up));
                    });
            }

            base.Initialize();
        }

        protected override void LoadContent()
        {
            mSpriteBatch = new SpriteBatch(GraphicsDevice);
            mFont = Content.Load<SpriteFont>("Tahoma");
        }

        protected override void Update(GameTime gameTime)
        {
            IInputManager input 
                = (IInputManager)Services.GetService(typeof(IInputManager));

            mMouseX = input.X;
            mMouseY = input.Y;
            mMouseZ = input.Z;

            mMouseDX = 0;
            mMouseDY = 0;
            mMouseDZ = 0;

            while (mKeyEvents.Count > 30)
                mKeyEvents.RemoveAt(0);

            while (mMouseEvents.Count > 30)
                mMouseEvents.RemoveAt(0);

            base.Update(gameTime);
        }

        protected override void Draw(GameTime gameTime)
        {
            mGraphics.GraphicsDevice.Clear(Color.Black);

            mSpriteBatch.Begin();

            mSpriteBatch.DrawString(mFont, string.Format("X: {0}", mMouseX),
                new Vector2(10f, 10f), Color.White);

            mSpriteBatch.DrawString(mFont, string.Format("Y: {0}", mMouseY),
                new Vector2(10f, 25f), Color.White);

            mSpriteBatch.DrawString(mFont, string.Format("Wheel: {0}", mMouseZ),
                new Vector2(10f, 40f), Color.White);

            mSpriteBatch.DrawString(mFont, string.Format("Delta X: {0}", mMouseDX),
                new Vector2(10f, 60f), Color.White);

            mSpriteBatch.DrawString(mFont, string.Format("Delta Y: {0}", mMouseDY),
                new Vector2(10f, 75f), Color.White);

            mSpriteBatch.DrawString(mFont, string.Format("Delta Wheel: {0}", mMouseDZ),
                new Vector2(10f, 90f), Color.White);

            float ystart = 10f;
            float yinc = 15f;

            for (int i = 0; i < mKeyEvents.Count; ++i)
            {
                KeyInputEvent evt = mKeyEvents[i];
                mSpriteBatch.DrawString(mFont, string.Format("{0} {1}", evt.Key.ToString(),
                    evt.Type.ToString()), new Vector2(200f, ystart + (i * yinc)), 
                    new Color(Vector3.One * ((float)i / (float)mKeyEvents.Count)));
            }

            for (int i = 0; i < mMouseEvents.Count; ++i)
            {
                MouseInputEvent evt = mMouseEvents[i];
                mSpriteBatch.DrawString(mFont, string.Format("{0} {1}", evt.Button.ToString(),
                    evt.Type.ToString()), new Vector2(400f, ystart + (i * yinc)),
                    new Color(Vector3.One * ((float)i / (float)mMouseEvents.Count)));
            }

            mSpriteBatch.End();

            base.Draw(gameTime);
        }
    }
}
